using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using SecretLabs.NETMF.Hardware.NetduinoPlus;

namespace Nevyn.co.uk
{
    class ShiftRegister
    {
        #region Private Variables

        /// <summary>
        /// Array containing the bits to be output to the shift register.
        /// </summary>
        private bool[] _bits;

        /// <summary>
        /// Number of chips required to implement this ShiftRegister.
        /// </summary>
        private int _numberOfChips;

        /// <summary>
        /// SPI interface used to communicate with the shift registers.
        /// </summary>
        private SPI _spi;

        #endregion

        #region Constructor(s)

        /// <summary>
        /// Default constructor.
        /// </summary>
        private ShiftRegister()
        {
            _bits = null;
            _spi = null;
        }

        /// <summary>
        /// Constructor for the ShiftRegister.
        /// </summary>
        /// <param name="bits">Number of bits in the shift register (should be a multiple of 8 bits).</param>
        public ShiftRegister(int bits, Cpu.Pin latchPin = Cpu.Pin.GPIO_Pin8, SPI.SPI_module spiModule = SPI.SPI_module.SPI1, uint speedKHz = 10)
        {
            if ((bits > 0) && ((bits % 8) == 0))
            {
                _bits = new bool[bits];
                _numberOfChips = bits / 8;
                for (int index = 0; index < bits; index++)
                {
                    _bits[index] = false;
                }

                SPI.Configuration config;

                config = new SPI.Configuration(SPI_mod: spiModule, ChipSelect_Port: latchPin,
                                               ChipSelect_ActiveState: false, ChipSelect_SetupTime: 0,
                                               ChipSelect_HoldTime: 0, Clock_IdleState: true, Clock_Edge: false,
                                               Clock_RateKHz: speedKHz);

                _spi = new SPI(config);
            }
            else
            {
                throw new ArgumentOutOfRangeException("ShiftRegister: Size must be greater than zero and a multiple of 8 bits");
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Overload the index operator to allow the user to get/set a particular 
        /// bit in the shift register.
        /// </summary>
        /// <param name="bit">Bit number to get/set.</param>
        /// <returns>Value in the specified bit.</returns>
        public bool this[int bit]
        {
            get
            {
                if ((bit >= 0) && (bit < _bits.Length))
                {
                    return (_bits[bit]);
                }
                throw new IndexOutOfRangeException("ShiftRegister: Bit index out of range.");
            }
            set
            {
                if ((bit >= 0) && (bit < _bits.Length))
                {
                    _bits[bit] = value;
                }
                else
                {
                    throw new IndexOutOfRangeException("ShiftRegister: Bit index out of range.");
                }
            }
        }

        /// <summary>
        /// Send the data to the SPI interface.
        /// </summary>
        public void LatchData()
        {
            byte[] data = new byte[_numberOfChips];

            for (int chip = 0; chip < _numberOfChips; chip++)
            {
                data[chip] = 0;
                byte bitValue = 1;
                int offset = chip * 8;
                for (int bit = 0; bit < 8; bit++)
                {
                    if (_bits[offset + bit])
                    {
                        data[chip] |= bitValue;
                    }
                    bitValue <<= 1;
                }
            }
            _spi.Write(data);
        }

        #endregion
    }
}
